import * as solid_js from 'solid-js';
import { JSXElement, Accessor } from 'solid-js';
import { EffectScope, Ref, UnwrapRef, UnwrapNestedRefs } from '@vue/runtime-core';

declare const enum LifecycleHooks {
    BEFORE_MOUNT = "bm",
    MOUNTED = "m",
    BEFORE_UPDATE = "bu",
    UPDATED = "u",
    BEFORE_UNMOUNT = "bum",
    UNMOUNTED = "um",
    PROPS_UPDATED = "pu"
}
interface ReactivueInternalInstance {
    scope: EffectScope;
    provides: Record<string | symbol, any>;
    parent: any;
    isMounted?: boolean;
    isUnmounted?: boolean;
    active: boolean;
    [LifecycleHooks.BEFORE_MOUNT]?: (() => void)[];
    [LifecycleHooks.MOUNTED]?: (() => void)[];
    [LifecycleHooks.BEFORE_UPDATE]?: (() => void)[];
    [LifecycleHooks.UPDATED]?: (() => void)[];
    [LifecycleHooks.BEFORE_UNMOUNT]?: (() => void)[];
    [LifecycleHooks.UNMOUNTED]?: (() => void)[];
    [LifecycleHooks.PROPS_UPDATED]?: ((props: any) => void)[];
}

declare type AccessorState<T> = {
    [K in keyof T]: T[K] extends Function ? T[K] : T[K] extends Ref<infer V> ? Accessor<UnwrapRef<V>> : T[K] extends UnwrapNestedRefs<infer A> ? Accessor<A> : T[K];
} & {
    setup: T;
};
declare global {
    interface Window {
        __solidivueRootInstance: ReactivueInternalInstance;
    }
}
declare type ReturnedSetup<T> = T extends Ref<unknown> ? Accessor<UnwrapRef<T>> : AccessorState<T>;
declare function useSetup<State, Props = {}>(fn: (props: Props) => State, SolidProps?: Props): ReturnedSetup<State>;
/**
 * @description `defineComponent` can't preverse state across HMR updates in `@refresh granular` mode.
 */
declare function defineComponent<PropsType, State>(setupFunction: (props: PropsType) => State, renderFunction: (state: ReturnedSetup<State>, props: PropsType) => JSXElement): (props: PropsType) => JSXElement;
/**
 * @description `createSetup` can't preverse state across HMR updates in `@refresh granular` mode.
 */
declare function createSetup<PropsType, State>(setupFunction: (props: PropsType) => State): (props: PropsType) => ReturnedSetup<State>;
declare function ReactivueProvider({ plugins, children }: {
    plugins?: any[];
    children?: JSXElement | JSXElement[];
}): number | true | Node | solid_js.JSX.ArrayElement | solid_js.JSX.FunctionElement | (string & {}) | undefined;

export { ReactivueProvider, createSetup, defineComponent, useSetup };
